'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _required = require('./helpers/required');

var _required2 = _interopRequireDefault(_required);

var _NotImplementedUseCase = require('./NotImplementedUseCase');

var _NotImplementedUseCase2 = _interopRequireDefault(_NotImplementedUseCase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var EntryPoint = function () {
  function EntryPoint(_ref) {
    var _ref$config = _ref.config,
        config = _ref$config === undefined ? (0, _required2.default)('config') : _ref$config,
        _ref$useCases = _ref.useCases,
        useCases = _ref$useCases === undefined ? (0, _required2.default)('useCases') : _ref$useCases;
    (0, _classCallCheck3.default)(this, EntryPoint);

    this._config = config;
    this._useCases = useCases;
  }

  /**
   * Retreives one entry from the public dictionary
   * The dictionary will contain the use cases and configuration
   * @example
   * const configuration = domain.get('config')
   * const useCase = domain.get('phone_contact_use_case')
   * @param {string} key The name of the use case or indicate that we want to get the config
   * @return (config|UseCase)
   */


  (0, _createClass3.default)(EntryPoint, [{
    key: 'get',
    value: function get(key) {
      // config key is still sync and we should return as it
      if (key === 'config') {
        return this._config;
      }

      // get the loader and the method from the useCases

      var _ref2 = this._useCases[key] || [],
          _ref3 = (0, _slicedToArray3.default)(_ref2, 2),
          loader = _ref3[0],
          method = _ref3[1];
      // if loader is undefined then is not implemented, otherwhise load async the useCase


      return loader === undefined ? new _NotImplementedUseCase2.default(key) : {
        execute: function execute(params) {
          // load async the factory, execute use case and return the promise
          return loader().then(function (factory) {
            return factory.default[method]().execute(params);
          });
        },
        $: {
          execute: {
            subscribe: function subscribe(fn) {
              // creating an object here without an empty dispose function
              var ret = { dispose: function dispose() {} };
              loader().then(function (factory) {
                // black magic: mutate the object, very small memory leak but that
                // makes dispose working async and we need it
                ret.dispose = factory.default[method]().$.execute.subscribe(fn).dispose;
              });
              // return the object that will be mutated async
              return ret;
            }
          }
        }
      };
    }
  }, {
    key: 'config',
    value: function config(key, value) {
      this._config.set(key, value);
      return this;
    }
  }]);
  return EntryPoint;
}();

exports.default = EntryPoint;